// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: ArfTableIterator.h
// Author: Hassan Siddiqui
//
// Description: 
// ARF generation task for the XMM Science Analysis System
// 
// Comments: 
//  This prototype is restricted the central CCD of the MOS1 detector.
//  
// Modifications: See ChangeLog
// 
// $Id: ArfTableIterator.cc,v 1.2 2005/12/04 21:30:53 rsaxton Exp $

/* General */

/* SAS-specific */
#include "Dal.h"
#include "Tracer.h"

/* Local classes */
#include "ArfTableIterator.h"
#include "AreaCalc.h"

/** Constructors */
ArfTableIterator::ArfTableIterator():
    _elowCol(0),
    _ehighCol(0),
    _prodCol(0)
    {
      PROC;
      ENDPROC;
    }

/** Destructors */
ArfTableIterator::~ArfTableIterator()
    {
      PROC;
      ENDPROC;
    }

/* Initialise table */
void ArfTableIterator::table( Table * )
    {
      PROC;

      /*
       * Don't do anything until subtable iteration implemented 
       * (and SubTable::addColumn method complete)
       * use temporary setupTable() method.
       */

      ENDPROC;
    }

  void ArfTableIterator::setupTable( Table * table )
    {
      PROC;

      _elowCol  = table->addColumn("ENERG_LO", 
				   Column::Real32, 
				   "No comment", 
				   "keV");

      _ehighCol = table->addColumn("ENERG_HI", 
				   Column::Real32, 
				   "No comment", 
				   "keV");

      _prodCol =  table->addColumn("SPECRESP", 
				   Column::Real32, 
				   "No comment", 
				   "cm2");
    }

  /** Subtable Iteration */

  void ArfTableIterator::processEntireTable(Table* table)
    {
      PROC;
      
      setupTable(table);

      /* Access column information */

      /* !!! Remember to delete ColumnData objects */
      ColumnData * elowColData      = _elowCol->data();
      real32     * elowData         = elowColData->real32Data();
      
      ColumnData * ehighColData     = _ehighCol->data();
      real32     * ehighData        = ehighColData->real32Data();

      ColumnData * prodColData      = _prodCol->data();
      real32     * prodData         = prodColData->real32Data();
      
      /* Create an AreaCalc object */
      AreaCalc areaCalc;

      /* Generate vector of energies to use in areaCalc.area() call,
         as well as set elow/ehigh columns in o/p dataset. */
      vector<double> energyVector;

      errstr << "Writing energy data into dataset .. "
	     << Sas::message(Msg::App,Msg::Sparse);

      for (unsigned int energy_index = 0;
	   energy_index < _elowCol->rows();
	   ++energy_index)
	{

	  /* Copy elow/ehigh data to arf columns. */
	  Interval<double> ebins  
	    = arfDataServer->ebins()->bin(energy_index);
      
	  /* Remember to convert to keV */
	  elowData[energy_index]  = ebins.lo()/1000.;
	  ehighData[energy_index] = ebins.hi()/1000.;

	  /* Get mean energy, in keV */
	  energyVector.push_back(0.5 * (ebins.lo() + ebins.hi()));
	}

      /* Get area vector */
      errstr 
	<< "Calculating Area curve ..."
	<< Sas::message(Msg::App, Msg::Sparse);

      errstr 
	<< "Calculating Area curve ..."<<energyVector[0]<<" "<<_elowCol->rows()
    <<" "<<energyVector[_elowCol->rows()-1]
	<< Sas::message(Msg::App, Msg::Sparse);

      vector<double> areaVector = areaCalc.area(energyVector);

      errstr << "Writing area data into dataset .. "
	     << Sas::message(Msg::App,Msg::Sparse);

      for (unsigned int energy_index = 0;
	   energy_index < _elowCol->rows();
	   ++energy_index)
	{
          if (energy_index > areaVector.size())
          {
             errstr<<"Anomaly in ArfTableIterator - raise SPR"
             << Sas::error("ARFObjError");
          }

	  errstr << "area[" << energy_index
		 << "]="
		 << areaVector[energy_index]
		 << Sas::message(Msg::App,Msg::Verbose);
	  
 	  prodData[energy_index]  = areaVector[energy_index];
	}

      /* Deleting ColumnData objects */
      delete elowColData;  elowColData  = 0;
      delete ehighColData; ehighColData = 0;
      delete prodColData;  prodColData  = 0;

      ENDPROC;
    }
