// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: CompactTablePixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Class representation of an iterator that handles DAL-based maps
// in a table format, but data are stored in such a way that a single row may
// represent one or many pixels.
//
// NOTE: This class assumes that
// a) There is a special row of int16-type that describes the number of pixels
// per row.
// b) y-values in pixel set of a given row goes as y0 +
// (N-1), where N is the number of pixels, y0 is the initial value
// stored in that row.
//
//
// Modifications: See ChangeLog
// 
// $Id: CompactTablePixelIterator.cc,v 1.1.1.1 2004/02/09 14:54:12 eojero Exp $

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"

// Local classes
#include "ArfDataServer.h"
#include "CompactTablePixelIterator.h"
#include "PositionUtils.h"

/* Constructors */
CompactTablePixelIterator::CompactTablePixelIterator(const string & tablename,
					     const string & xcolname,
					     const string & ycolname,
					     const string & valuecolname,
					     const string & countercolname) :
  _firstcalled(false),
  _set(0),
  _table(0),
  _xColData(0),
  _yColData(0),
  _valueColData(0),
  _counterColData(0),
  _xData(0),
  _yData(0),
  _valueData(0),
  _counterData(0),
  _wcsconv(0)
{
  PROC;
  
  _set = dataSetServer->
    open( tablename, 
	  DataSet::Read );

  _table = _set->table(tablename); 

  _tableelements = _table->rows();
  
  _xColumn     = _table->column(xcolname);
  _yColumn     = _table->column(ycolname);
  _valueColumn = _table->column(valuecolname);

  _xColData     = _xColumn    ->data();
  _yColData     = _yColumn    ->data();
  _valueColData = _valueColumn->data();

  /* If the counter column hasn't been specified assume that it is one */
  if (countercolname.empty()) {
 errstr<<"Setting counter col to 1 - elements: "<<_tableelements
         << Sas::message(Msg::App, Msg::Verbose);
   if (_tableelements > 0) {  
    for (unsigned int i=0; i<_tableelements; i++) {
        _tempcounter.push_back(1);
    }
    _counterData = &_tempcounter[0];

   }
  } else {
    _counterColumn = _table->column(countercolname);
    _counterColData = _counterColumn->data();
    _counterData  = _counterColData->int16Data();
  }

  _xData        = _xColData->int16Data();
  _yData        = _yColData->int16Data();
  _valueData    = _valueColData->int16Data();

  /* Determine _xindexsize and _yindexsize from data. 
   * Also check whether data are valid.
   */

  _xindexsize=_xData[0];
  _yindexsize=_yData[0];

  /* Create WCS convertor object */
  _wcsconv = new WCSConvertor(_xColumn, _yColumn);

  errstr << "Set up CompactTableIterator on table: " <<tablename
         << Sas::message(Msg::App, Msg::Verbose);
   
  ENDPROC;
}

/* Destructor */
CompactTablePixelIterator::~CompactTablePixelIterator()
{
  PROC;
  
  delete _xColData;      _xColData=0;
  delete _yColData;      _yColData=0;
  delete _valueColData;  _valueColData=0;
  delete _counterColData;  _counterColData=0;
  dataSetServer->close(_set); _set = 0;
 
  ENDPROC;
}

/* Iterator methods */
void CompactTablePixelIterator::first()
{
  /* Map the geometry the first time that the iterator is used */
  if (! _firstcalled) 
  {  
     _putils = arfDataServer->positionUtils();
     _detPixWidths = _putils->detxyWidth();

   /* Set up dummy WCS convertor to pass widths */
     //_wcsconv = new WCSConvertor("DETX", 0, 0, _detPixWidths[0], 
      //                           "DETY", 0, 0, _detPixWidths[1]);
  }

  _firstcalled = true;
  _index=0;

  /* Reset counter and set counterlimit to n of pixels in current row */
       errstr<<"Counter limit: "<<_counterData[_index]
         << Sas::message(Msg::App, Msg::Verbose);
  _pixelcounterlimit = _counterData[_index];
  _pixelcounter=0;

}


void CompactTablePixelIterator::next ()
{
  if (isDone()) 
  {
    errstr<<"Anomaly in CompactTablePixelIterator - raise SPR"
          << Sas::error("ARFObjError");
  }

  if ( ++_pixelcounter > (_pixelcounterlimit-1) )
    {
      _index++;

      /* Reset counter and set counterlimit to n of pixels in current row */
      _pixelcounterlimit = _counterData[_index];
      _pixelcounter=0;
    }
}

bool CompactTablePixelIterator::isDone()
{
  PROC;

    if (!_firstcalled)
    {
      errstr<<"Anomaly in CompactTablePixelIterator - raise SPR"
          << Sas::error("ARFObjError");
    }

  RETURN (_index >= _tableelements);
}

PixelElement CompactTablePixelIterator::currentItem()
{
  PROC;

    if (!_firstcalled)
    {
      errstr<<"Anomaly in CompactTablePixelIterator - raise SPR"
          << Sas::error("ARFObjError");
    }

  /* Compute yindex, as this is dependent 
     on number of pixels in compacted row 
     in addition to the y-value of that row */
  int ydata = _yData[_index] + _pixelcounter;

  double value = _valueData[_index];

  /* Calculate the detector coordinates for this pixel. This makes the
     class specific to raw pixels - we'll worry about this later! */
  PixCoords pixCoord1(_xData[_index], ydata);
  CamCoords camCoord2 = _putils->PixCoordTocamCoord2(pixCoord1);
  CoordPair detcen = _putils->CamCoord2Todetxy005(camCoord2);
  double det_xcentre = detcen.x();
  double det_ycentre = detcen.y();

  errstr << "CurrentItem X: "<<_xData[_index]<<" Y: "<<ydata
         << "Xdet: "<<det_xcentre<<"Ydet: "<<det_ycentre
         << Sas::message(Msg::App, Msg::Noisy);
   
  /* Return a pixel element which is a single raw pixel with equivalent
     detector coordinate position and width in det coords */
  PixelElement result (_xData[_index],
		       ydata,
		       det_xcentre,
		       det_ycentre,
		       _detPixWidths[0],
		       _detPixWidths[1],
		       value);

  RETURN result;
}

unsigned CompactTablePixelIterator::size() const
{
  PROC;

   /* Calculate the number of bad pixels in the extension */
    int totsize=0;
    for (unsigned int i=0; i < _tableelements; i++)
    {
       totsize +=_counterData[i];
    }

  RETURN totsize;
}

unsigned CompactTablePixelIterator::xbins() const
{
  PROC;
  
  RETURN _xindexsize;
}


/* WCSConvertor object */
const WCSConvertor * CompactTablePixelIterator::wcsconvertor() const
{
  PROC;
  RETURN _wcsconv;
}

