// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: ConcreteArfDataServer
// Author: Hassan Siddiqui
//
// Description: Concrete implementation of ArfDataServer class
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: ConcreteArfDataServer.h,v 1.11 2014/04/03 13:37:51 rsaxton Exp $

#ifndef ConcreteArfDataServer_H
#define ConcreteArfDataServer_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include "CalServer.h"
#include "Dal.h"
#include "Tracer.h"
#include "DataSubspace.h"

/* Local Classes */
#include "DatafileAttributes.h"
#include "CoreData.h"
#include "GeneralIterator.h"
#include "PsfModel.h"
#include "PhotonDistribution.h"
#include "Frame.h"
#include "DetectorPosition.h"
#include "PixelIterator.h"
#include "EeCorrector.h"
#include "BadPixelCorrector.h"
#include "Ebins.h"
#include "SourcePosition.h"
#include "PixCoords.h"
#include "CamCoords.h"
#include "TelCoords.h"
#include "PosWCS.h"
#include "DetmapFilter.h"
#include "RegionExtent.h"
#include "RegionFunctions.h"
#include "CrossRegionArf.h"

/****************************************************
 *
 * class ConcreteArfDataServer
 *
 ***************************************************/

class ConcreteArfDataServer
{
public:

  ConcreteArfDataServer();

  ~ConcreteArfDataServer();

  /********************************************
   *
   * General observation/mission/instrument info
   *
   ********************************************/

  Xmm::Instrument instrumentID()   const;
  string filterName()   const;


  /**************************
   *
   * Pointing info
   *
   **************************/

  /* Source position */
  double                     raref()                   const ;
  double                     decref()                  const ;

  bool                       useODF()                  const ;
  unsigned long              eventFlags()   	       const ;
  bool                       ignoreEdgePixels()        const ;
  bool                       ignoreOutofFOV()          const ;
/* AI gcc4 porting
 * error: type qualifiers ignored on function return type
 * => qualifiers removed
 */
  unsigned long       ccdMask()                 const ;

  void                 readDSS()                       ;

  /********************************************
   *
   * Selection info
   *
   ********************************************/
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => qualifiers removed
   */
  const DataSubspace *        dataSubspace()          const;
  const RegionExtent * extent() const;

  RegionExtent * addDetRegion( PositionUtils * putils );


  /********************************************
   *
   * Ebins accessor
   *
   ********************************************/
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => qualifiers removed
   * const XXXXX * const yyy()   const;
   * const XXXXX *       yyy()   const;
   */
  const BinnedAxis *           ebins()   const;

  /********************************************
   *
   * Source Position
   *
   ********************************************/
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => qualifiers removed
   * const XXXXX * const yyy()   const;
   * const XXXXX *       yyy()   const;
   */
  const SourcePosition *   sourcePosition() const;

  /********************************************
   *
   * Misc
   *
   ********************************************/

  DatafileAttributes * specAttributes()  const;

  bool filterDSS ()    const;

  const string filteredTableName ()    const;

  bool keepFilteredTable ()    const ;

  bool isExtendedSource() const;
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => qualifiers removed
   * const XXXXX * const yyy()   const;
   * const XXXXX *       yyy()   const;
   */
  const PhotonDistribution *   photonDistibution() const;

  const Frame *  frame()  const;
  /* AI gcc4 porting
    * error: type qualifiers ignored on function return type
    * => qualifiers removed
    * const XXXXX * const yyy()   const;
    *  XXXXX *       yyy()   const;
    */
   PsfModel *   psf() const;
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => qualifiers removed
   * const XXXXX * const yyy()   const;
   *  XXXXX *       yyy()   const;
   */
  PixelIterator *  detmapPixelIterator();
  /* AI gcc4 porting
     * error: type qualifiers ignored on function return type
     * => qualifiers removed
     * const XXXXX * const yyy()   const;
     *  XXXXX *       yyy()   const;
     */
   PixelIterator *  filteredBSPixelIt();
  /* AI gcc4 porting
    * error: type qualifiers ignored on function return type
    * => qualifiers removed
    * const XXXXX * const yyy()   const;
    *  XXXXX *       yyy()   const;
    */
   PixelIterator *  PsfPixelIterator();
/* AI gcc4 porting
 * end
 */
  double framePeriod() const;

  bool setCcd( const CamCoords & camcoord);
  /* AI gcc4 porting
   * error: extra qualification ‘ConcreteArfDataServer::’ on member ‘xxxxx’
   */
  int backScale() const;

  int sourceBoxArea() const;

  bool crossRegion() const;

  void set_bpcorrectors();
  /* AI gcc4 porting
   * end
   */

  /***********************
   *  Conversion routines
   ***********************/

  const DetectorPosition detxyToPixcoord( int detx,
					  int dety) const;

  /* Convert from CAMCOORD2 to PIXCOORD */
  const PixCoords camCoord2ToPixCoord( const CamCoords & camcoords ) const;

  /* Convert from DETX/DETY (0.05 arcsec units) to camcoord2 */
  const CamCoords detxy005ToCamCoord2(int detx_005,
				      int dety_005) const;

  /* Convert from CAMCOORD2 to TELCOORD */
  const TelCoords camCoord2ToTelCoord( const CamCoords & camCoords ) const;

  /* Return the positionUtils object */
  const PositionUtils * positionUtils() const;

  /* Convert from X/Y to DETX/DETY */
  SourcePosition xyToDetxy (double x, double y) const;

  /***********************
   *  Arf quantity routines
   ***********************/

  /* These methods can be turned on/off by user */
  EffectiveAreaCurve effectiveAreaCurve  ( double theta, double phi ) const;

  CcdQuantumEfficiencyCurve quantumEfficiencyCurve ( double  x ,
						     double  y ) const;

  FilterTransmissionCurve filterTransmissionCurve ( double  theta ,
						    double  phi ) const;

  ContaminantTransmissionCurve * contaminantTransmissionCurve () const;

  double pileupCorrection   (double  energy,
			     double  pixelFlux) const;

  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => qualifiers removed
   * const XXXXX * const yyy()   const;
   * const XXXXX *       yyy()   const;
   */
  const EeCorrector *   eeCorrector () const;

  const BpCorrector *   badPixCorrector () const;
/* AI gcc4 porting
 * end
 */

  double ootCorrection ( ) const;

  GeoRegion PolyRegion () const;

private:

  void createDetmap();
  void checkRegionExtent();
  void geoExtent(GeoRegion &region);
  bool standardDetMap(const string & arrayname);

  /* Member fields */

  DataSet              * _rmfset;
  Table                * _rmftable;

  DatafileAttributes   * _specAttributes;

  /* Detmap pointers */
  PixelIterator        * _rawDetmap;
  bool                 _filteredDetmapExists;
  PixelIterator        * _filteredDetmap;

  CoreData             * _coreData;

  Xmm::Instrument       _instrumentID;
  string               _instrument;
  string               _filter;
  string               _ppsvers;

  double               _raref;
  double               _decref;

  const PhotonDistribution   * _photonDistribution;
  const PsfModel             * _psf;
  const Frame                * _frame;

  bool                 _extendedsource;
  bool                 _modelEffectiveArea;
  bool                 _modelQuantumEfficiency;
  bool                 _modelFilterTransmission;
  bool                 _modelContaminantTransmission;
  bool                 _modelGratingAttenuation;
  bool                 _modelContamination;
  bool                 _modelootcorr;
  bool		       _applyxcaladjustment;
  bool                 _useodf;
  bool                 _crossRegion;

  const EeCorrector         * _eeCorrector;
  const BpCorrector   * _badPixCorrector;
  const BinnedAxis    * _ebins;

  DataSubspace  * _dss;

  const RegionExtent  * _extent;

  bool               _filterdss;
  string             _filteredtablename;
  bool               _keepfilteredtable;

  const EffectiveAreaCurve * _nulleacurve;
  const CcdQuantumEfficiencyCurve * _nullqecurve;
  const FilterTransmissionCurve   * _nullftcurve;
  ContaminantTransmissionCurve   * _nullcncurve;

  const PosWCS *      _poswcs;

  const CoordPair *   _refcoords;

  PsfModel *          _bp_psf;

  DetmapFilter *      _detmapfilter;

  RegionFunctions *   _rfunction;

  CrossRegionArf * _crossRegionArf;

  unsigned            _sourceBoxArea;

  bool		      _regPosCoords;
  bool		      _regRawCoords;
  bool		      _TimOrBurstMode;
  bool		      _maskFound;


  Xmm::Mode           _windowMode;

  unsigned long       _flag;
  unsigned long       _ccdMask;

  PixelIterator * _filtBSPixelIt;
  string          _bstempfile; // Name of BS Map file

  double _fastmode_Yfraction; // Fraction of Y extent in Fast mode region
  double _psfscale; // Fraction to correct burst mode box size
};

#endif
