// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// 
// Module: CoreData
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: CoreData.h,v 1.5 2014/04/03 13:37:51 rsaxton Exp $

#ifndef CoreData_H
#define CoreData_H

/* General classes */
#include <iostream>

/* SAS-specific */
#include "Xmm.h"
#include "CalCommon.h"
#include "CalServer.h"
#include "CamCoords.h"
#include "DataSubspace.h"
#include "Region.h"

/* Local */
#include "SourcePosition.h"
#include "PositionUtils.h"
#include "GeneralIterator.h"
#include "PsfModel.h"
#include "PixCoords.h"
#include "TelCoords.h"
#include "PileupCorrector.h"
#include "RegionExtent.h"

class CoreData
{
public:

  /*  Constructors  */
  CoreData();

  /*  Destructors  */
  virtual ~CoreData();

  /* Change state of other accessors which depend on ccd number. */
  virtual bool setCcd(const CamCoords & camcoords) = 0;

  /* Get ebins
   * Note: object should be deleted by client of method */
  virtual const BinnedAxis * ebins() const = 0;

  /* Methods providing ccd ID range; 
   * useful for detector map subregion generation */
  virtual unsigned int ccdFirst() const = 0;
  virtual unsigned int ccdLast()  const = 0;

  /******************************************************
   * 
   * Geometry based routines
   *
   ******************************************************/

  /* Convert from DETX/DETY (0.05 arcsec units) to camcoord2 */
  virtual  const CamCoords detxy005ToCamCoord2( int detx_005, 
						int dety_005 ) const = 0;

  /* Convert from CAMCOORD2 to PIXCOORD */
  virtual const PixCoords camCoord2ToPixCoord( const CamCoords & camcoords ) const = 0;

  /* Convert from CAMCOORD2 to TELCOORD */
  virtual const TelCoords camCoord2ToTelCoord( const CamCoords & camcoords ) const = 0;

   /* Convert from X/Y to DETX/DETY */
  virtual SourcePosition xyToDetxy (double x, double y) const = 0;

  /* Get source position */
  virtual const SourcePosition * sourcePosition() const = 0;

  /* Set source position */
  virtual void setSourcePosition( const string & sourceCoordType,
                      double x, double y) = 0;

  /* Get position utilities */
  virtual const PositionUtils * positionUtils() const = 0;
  virtual PositionUtils * varpositionUtils() = 0;

  virtual PsfModel *      psf()        const = 0;

  /* Frame period, in seconds */
  virtual double framePeriod()          const = 0;

  
  /* Iterator functions */

  virtual GeneralIterator<int> * ccdIterator()          = 0; 

  virtual GeneralIterator<int> * xpixelIterator() const = 0; 

  virtual GeneralIterator<int> * ypixelIterator() const = 0;


  /* Data resource functions */


  virtual EffectiveAreaCurve effectiveArea () const = 0;

  virtual EffectiveAreaCurve effectiveArea ( double theta, 
                          double phi ) const = 0;

  virtual EffectiveAreaCurve mirrorAdjustment(
                              EffectiveAreaCurve eain) const = 0;

  virtual CcdQuantumEfficiencyCurve 
  quantumEfficiencyCurve ( double  x,
			   double  y ) const = 0;

  virtual FilterTransmissionCurve
  filterTransmissionCurve     ( double  theta , 
				double  phi ) const = 0;

  virtual ContaminantTransmissionCurve *
    contaminantTransmissionCurve     () const = 0;

  
  //  virtual double gratingAttenuation     ( double  energy ) const = 0;

  virtual const PileupCorrector * pileupCorrector() const = 0;

  /* Initialise patterns from DSS */
  virtual void initPatterns( DataSubspace * dss ) = 0;
  virtual void checkModePattern( Xmm::Mode windowMode ) = 0;

  virtual void setPSF( const GeoRegion & region, const RegionExtent * extent ) =0;
  virtual void setPSF_FMF( double factor ) = 0;
  virtual void setSRCPOS( int SRCPOS ) = 0;
};
#endif
