// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: DetmapPsfModel
// Author: Richard Saxton
//
// Description: 
//
// Sets up a PSF model based upon a detector map which will have come from 
// an input image. The input image giving the spatial distribution of the
// emission may come from another instrument or satellite
//
// Modifications: See ChangeLog
// 
// General classes
#include <iostream>
#include <cmath>

// SAS-specific classes
#include "Region.h"

// General utility functions
#include "errstr.h"
#include "Tracer.h"
#include "NumericalRange.h"

// Local classes
#include "DetmapPsfModel.h"

/* Constructor */
//
// imagePixItr - should be a normalised image
DetmapPsfModel::DetmapPsfModel( PixelIterator * imagePixItr)
{
  PROC;

   // Normalise here rather than in iterator
   //imagePixItr->normalise();
   _norm = 0.0;

   // Create an image to contain the iterator
   _imin = imagePixItr->physXYmin();
   _imax = imagePixItr->physXYmax();
   _xwid = imagePixItr->wcsconvertor()->xWidth();
   _ywid = imagePixItr->wcsconvertor()->yWidth();
   _nx = (_imax.x() - _imin.x()) / _xwid + 1;
   _ny = (_imax.y() - _imin.y()) / _ywid + 1;

   //cout<<"Im size: "<<_imin.x()<<" "<<_imin.y()<<" "<<_imax.x()<<" "<<_imax.y()
    //   <<" "<<_xwid<<" "<<_ywid<<" "<<_nx<<" "<<_ny<<endl;

   _dIm = new SImage(_nx,_ny);
   SImage &dIm = *_dIm;

   // Read each iterator pixel
   FOREACH(imagePixItr)
    {
       PixelElement p = imagePixItr->currentItem();

       // Find image pixel position
       int ix=int((p.xCentre() - _imin.x()) / _xwid);
       int iy=int((p.yCentre() - _imin.y()) / _ywid);

       //cout<<"Sett: "<<p.xCentre()<<" "<<ix<<" "<<p.yCentre()<<" "<<iy<<" "<<p.value()<<endl;

       dIm[ix][iy]+=float(p.value());

       _norm+=p.value();
       //cout<<"Map pix: "<<ix<<" "<<iy<<" "<<dIm[ix][iy]<<endl;
    }

   errstr 
    << "DetmapPsfModel: constructed with "<< imagePixItr->size()<<" pixels"
    << " using intermediate image of dimension "<<_nx<<" "<<_ny<<" and normalisation"
    << _norm<<Sas::message(Msg::App, Msg::Verbose);

  ENDPROC;
};

DetmapPsfModel::~DetmapPsfModel()
{
  PROC;
  errstr 
    << "DetmapPsfModel destroyed."
    << Sas::message(Msg::App, Msg::Verbose);

  ENDPROC;

}

/* Return the fraction of the total represented by this box */

double DetmapPsfModel::int_psf( double xboxcentre, 
			  double yboxcentre, 
			  double  xboxsize , 
			  double  yboxsize  ) const
{
  PROC;

    // Find X and Y range
     //double ee = xboxsize * yboxsize / _totalpixels;
     double ee=1.0/_norm; // FIX!!!!!!!!!!!!!!!!!!!!!!!!!

     errstr 
  	<< "Enboxed energy: "<<ee
  	<< Sas::message(Msg::App, Msg::Noisy);

  RETURN ee;
}

// Calculate the enclosed area from a set of pixels as a fraction
// of the total area
double DetmapPsfModel::enclosed_psf( PixelIterator * pxitr ) const
{
  PROC;

  double ee = 0.0;

  SImage &dIm = *_dIm; // Use reference so that we can treat as 2-d array

  // Find the sum of the detector map values which coincide with the pixels
  // in the iterator. Normalise by pixel area which has to be done individually
  // as pixel widths can vary.
  FOREACH(pxitr)
    {
      PixelElement dpix = pxitr->currentItem();
      
      // Convert from the pixel detector coordinates to image pixel position 
      int ix = int((dpix.xCentre() - _imin.x()) / _xwid);
      int iy = int((dpix.yCentre() - _imin.y()) / _ywid);

      // Sum normalised detector map pixel value if within image bounds
      if (ix>=0 && ix <_nx && iy>=0 && iy<_ny) {
         //cout<<"Adding elem: "<<dpix.xCentre()<<" "<<dpix.yCentre()<<" "<<dpix.value()
          //<<" "<<ix<<" "<<iy<<" "<<_imin.x()<<" "<<_imin.y()<<" "<<dIm[ix][iy]<<endl;
          ee += double(dIm[ix][iy]) * dpix.xWidth() * dpix.yWidth();
      }
    }

    // Normalise the sum by the total image normalisation and the total image size
    //cout<<"Initial ee: "<<ee<<" "<<_norm<<" "<<_xwid<<" "<<_ywid<<endl;

    ee = ee / _norm / (_xwid * _ywid);
      
    errstr
        << "Enclosed energy: "<<ee
        << Sas::message(Msg::App, Msg::Verbose);

  RETURN ee;
}

void DetmapPsfModel::energy(double /* energy */)
{
  PROC; 
  ENDPROC;
}

void DetmapPsfModel::setRadius(vector<double> /* radii */)
{
  PROC; 
  ENDPROC;
}

void DetmapPsfModel::setNgridEls(unsigned int /* NGridEls */)
{
  PROC; 
  ENDPROC;
}

double DetmapPsfModel::psf(double /* x */,
                     double /* y */) const
{
  PROC; 

   /* Do nothing */
  RETURN 1.0;
}

string DetmapPsfModel::mode() const
{
  PROC; 

  RETURN "DETMAP";
}
double DetmapPsfModel::rad_psf() const
{
  PROC;
  RETURN 1.0;
}
void DetmapPsfModel::setBoxShape( ShapeBasics::ShapeType shape ) 
{
  PROC;

    errstr<<"Shape type "<<shape
    << Sas::message(Msg::App, Msg::Verbose);

  ENDPROC;
}
void DetmapPsfModel::setFastModeFactor(double factor)
{
  PROC;
                                                                                
    errstr<<"Fast mode factor "<<factor
    << Sas::message(Msg::App, Msg::Verbose);
                                                                                
  ENDPROC;
}
void DetmapPsfModel::setSRCPOS(int SRCPOS)
{
  PROC;

    errstr<<"SRCPOS: "<<SRCPOS
    << Sas::message(Msg::App, Msg::Verbose);

  ENDPROC;
}

